% P50SNRpileup3.m SNR with pileup-3
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2014. All rights reserved.
 
%% init vars
post_title = 'P50SNRpileup3';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
% MakeBlogPostFileLists(post_title);

%% Create the input spectrum and attenuation coefficients
tube_voltage = 80; % keV  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
materials = {'soft_tissue','bone_cortical'}; 
specdat.mus = MusFromMaterials(materials,specdat.egys);
spinv =  SpectrumCumulativeInverse(specdat);

%%    Monte Carlo parameters
mean_cps = 1e6; % 1/seconds
t_integrate = 1.5e-3; %sec
ndeadtimes = 10;
eta_max = 0.5;
deadtimes = linspace(0,eta_max/mean_cps,ndeadtimes);
  % compute the required number of input pulses so probability of
  % "running off end" during the integration time is small
n_pulses_ave = mean_cps*t_integrate;
dev_npulses = sqrt(n_pulses_ave); % input pulses are Poisson distributed so variance = mean
npulses2gen =  round(n_pulses_ave+50*dev_npulses); % number of pulses to generate to make sure have enough pulses
ntrials = 2000;    

%% simulate count and PHA data vs. deadtime
nqdat = zeros(ndeadtimes,ntrials,2);    % pre-allocate variable to hold MC data, 
            % second dim holds number of recorded counts and total energy pf photons during integrate period
  % prep parameters for PHA
specdat.Ethreshold = 55;
egy_bin_edges = [0 specdat.Ethreshold inf];
nbins = numel(egy_bin_edges)-1;
phacnts = zeros(ndeadtimes,ntrials,nbins);
statspha = zeros(ndeadtimes,2*nbins+1); % mean counts , variances and covariance of bins for each deadtime
fprintf('\n');
for kdead = 1:ndeadtimes
    deadtime = deadtimes(kdead);
    for ktrial = 1:ntrials
        dt_pulse = -(1/mean_cps)*log(rand(npulses2gen,1)); % exponentially distributed interpulse times
        [npulses_record, egys_rec] = PileupPulses(dt_pulse,deadtime, ...
            t_integrate,'specinv',spinv); 
            % PHA data
        counts = histc(egys_rec,egy_bin_edges);
        phacnts(kdead, ktrial, :) = counts(1:(end-1));
    end
    cv = cov(squeeze(phacnts(kdead,:,:)));
    statspha(kdead,:) = [squeeze(mean(phacnts(kdead,:,:)))', diag(cv)',cv(1,end)];
   fprintf('.')
   if mod(kdead,10)==0
       fprintf('\n');
   end
end
for k=1:5,beep,pause(0.1),end
fprintf('\ndone\n');

%% plot PHA mean counts and variances vs. theory
     % the parameters for the theoretical formulas
nbar_incident = mean_cps*t_integrate;
   % get bin fractions etc for spectra with pileup
etas = mean_cps*deadtimes(:);
fracs = zeros(ndeadtimes,nbins);
for kdead = 1:ndeadtimes
    deadtime = deadtimes(kdead);
    spout = SpectrumWithPileup(specdat,etas(kdead));
    frac = BinFracs(spout,specdat.Ethreshold);
    fracs(kdead,:) = frac(:)';
end
     % the theoretical results and the Monte Carlo simulation statistics
          % first the mean values of counts in each bin
nbars_MC = statspha(:,1:nbins); % mean counts in each bin for all deadtimes from the Monte Carlo data
nrecord_bar = nbar_incident./(1+etas); % theoretical mean total recorded counts M
     % nbars_calc is the theoretical mean counts for each bin for deadtimes
nbars_calc = repmat(nbar_incident./(1 + etas),1,nbins).*fracs(:,1:nbins);
     % then the variance
Ncov_MC = statspha(:,(nbins + 1):end); % variances of each bin and cov for all deadtimes from the Monte Carlo data
nrecord_var = nbar_incident./(1+etas).^3; % theoretical var of total recorded counts var(M)
           % theoretical variance of each bin for all deadtimes 
Nvars_calc = repmat(nrecord_bar,1,2).*fracs(:,1:nbins) + ... 
    repmat((nrecord_var - nrecord_bar),1,nbins).*(fracs(:,1:nbins).^2);
Ncov_calc = prod(fracs,2).*(nrecord_var - nrecord_bar);
 % plot the data
fh = figure;
hold on
    % results are normalized by dividing by the theoretical mean counts for each bin
plot(etas,nbars_MC(:,1)/nbar_incident,'ok',etas,nbars_calc(:,1)/nbar_incident,'-k','linewidth',1.5);
plot(etas,nbars_MC(:,2)/nbar_incident,'+r',etas,nbars_calc(:,2)/nbar_incident,'--r','linewidth',1.5);
plot(etas,Ncov_MC(:,1)/nbar_incident,'db',etas,Nvars_calc(:,1)/nbar_incident,'-b','linewidth',1.5);
plot(etas,Ncov_MC(:,2)/nbar_incident,'*g',etas,Nvars_calc(:,2)/nbar_incident,'--g','linewidth',1.5);
plot(etas,Ncov_MC(:,3)/nbar_incident,'xm',etas,Ncov_calc/nbar_incident,'-m','linewidth',1.5);
hold off
hl = legend('N1 Monte Carlo','N1 theory','N2 Monte Carlo','N2 theory', ...
    'var N1 Monte Carlo','var N1 theory','var N2 Monte Carlo','var N2 theory', ...
    'cov(N1,N2) Monte Carlo','cov(N1,N2) theory');
set(hl,'fontsize',10);
axis([0 0.5 -.1 1])
xlabel('mean_cps*deadtime','fontsize',14,'interpreter','none')
ylabel('avg recorded counts, variance and cov/incident counts','fontsize',14)
set(gca,'fontsize',14,'linewidth',1.5)
set(fh,'color','w','position',[250 400 600 600])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'PHAstats';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi
